local dynaskin_list = {}
local dynaskin_categories = {"Unsorted"}
/**
 * Adds a Custom DynaSkin to the list of all Dynaskins
 * Puts it in the Unsorted Category, if it is found to not have one already
 * If it is a cycle type, set a default cycle_speed unless otherwise stated
 */
rawset(_G,"Add_DynaSkin",function(added)
	if added.category == nil or not added.category then
		added.category = "Unsorted"
	end
	
	if added.cycle_speed == nil then
		added.cycle_speed = 10
	end
	table.insert(dynaskin_list,added)
end)

/**
 * Adds a Custom DynaSkin Category for displaying in lists
 */
rawset(_G, "Add_DynaCategory", function(added)
	table.insert(dynaskin_categories,added)
end)

/**
 * Controls all Logic behind the dynaskins command
 * Displays list of all dynaskins if none provided
 * If Dynaskin provided, give Player Dynaskin
 */
local function dynaskinsChange(p, arg)
	local valid_argument = false
	//If no argument given, list all possible dynaskins
	if arg == nil then
		CONS_Printf(p, "==\135D Y N A S K I N S\128==")
		for catkey,catvalue in ipairs(dynaskin_categories) do
			CONS_Printf(p,"\136--"+catvalue+"\136--")
			for key,value in ipairs(dynaskin_list) do
				if catvalue == value.category then
					CONS_Printf(p,"\135dynaskins\130 "+value.name+"\128 ("+value.desc+"\128)")
				end
			end
		end	
		CONS_Printf(p, "\n\135dynaskins\133 off\128 - (Disable any active dynaskin)")
		
		return
		
	//If an argument given, see if argument is valid
	else
		for key, value in ipairs(dynaskin_list) do
			if arg:lower() == value.name
				valid_argument = true
			end
		end
		
		if valid_argument == false and arg:lower() == "off" then
			valid_argument = true
		end
 	end
	//If the argument is valid, then set the dynaskin to the argument
	if valid_argument then
		p.initSkin = true
		p.dynaskin = arg
		p.blinking = false
	else
		CONS_Printf(p, "\133Invalid option")
	end
end
COM_AddCommand("dynaskins", dynaskinsChange, 0)

/**
 * Initialize the Skin when 1st Selected
 */
local function skin_init(p)
	if not p.mo.valid then return end
	local doSkin = p.dynaskin
	local selectedSkin = {}
	for key, value in ipairs(dynaskin_list) do
		if value.name == doSkin then
			selectedSkin.name = value.name
			selectedSkin.desc = value.desc
			selectedSkin.skin_init = value.skin_init
			selectedSkin.color = value.color
		end
	end
	
	if selectedSkin.skin_init then
		selectedSkin.skin_init(p, selectedSkin)
	elseif doSkin == "off" then
			p.mo.color = p.originalColor
	else
		p.dynaskin = "off"
	end
	p.expected = p.mo.color
end

/**
 * Control the Skin after Initialization
 */
local function doskin_think(p, skin)
	local selectedSkin = {}
	for key, value in ipairs(dynaskin_list) do
		if skin == value.name then
			selectedSkin.color = value.color
			selectedSkin.skin_think = value.skin_think
			selectedSkin.cycle_speed = value.cycle_speed
		end
	end

	if selectedSkin.skin_think then
		selectedSkin.skin_think(p, selectedSkin)
	end
	
	//If Using Super Color, p.skincolor can't be the same value... let's approximate
	if p.mo.color >= SKINCOLOR_SUPER1 then
		if p.mo.color <= SKINCOLOR_SUPER5 then
			p.skincolor = SKINCOLOR_YELLOW
		elseif p.mo.color <= SKINCOLOR_TSUPER5 then
			p.skincolor = SKINCOLOR_CREAMSICLE
		elseif p.mo.color <= SKINCOLOR_KSUPER5 then
			p.skincolor = SKINCOLOR_RED
		elseif p.mo.color <= SKINCOLOR_PSUPER5 then
			p.skincolor = SKINCOLOR_LILAC
		elseif p.mo.color <= SKINCOLOR_BSUPER5 then
			p.skincolor = SKINCOLOR_PERIWINKLE
		elseif p.mo.color <= SKINCOLOR_ASUPER5 then
			p.skincolor = SKINCOLOR_CYAN
		elseif p.mo.color <= SKINCOLOR_GSUPER5 then
			p.skincolor = SKINCOLOR_LIME
		elseif p.mo.color <= SKINCOLOR_WSUPER5 then
			p.skincolor = SKINCOLOR_WHITE
		elseif p.mo.color <= SKINCOLOR_CSUPER5 then
			p.skincolor = SKINCOLOR_CREAM
		end
	else
		p.skincolor = p.mo.color
	end
end

/**
 * Controls what happens to the skin (when to initialize and when to think)
 */
addHook("PlayerThink", function(p)
	if p == nil or p.mo == nil then return end
	
	if p.dynaskin == nil
		p.dynaskin = "off"
		p.expected = 0
	end
	
	if not p.cmd.buttons then	--they're in the menu
		if p.expected ~= p.mo.color and not p.initSkin
			p.menu = true
			p.originalColor = p.mo.color
			--p.skincolor = p.expected
		end
	else
		if p.menu == true
			p.menu = false
			p.initSkin = true
		end
	end
	
	if p.initSkin
		if p.originalColor == nil or p.originalColor == 0
			p.originalColor = p.mo.color
		end
		skin_init(p)
		p.initSkin = false
	end
	doskin_think(p, p.dynaskin)
	p.expected = p.mo.color	

end)

--------------------------------------------------------------------------------
rawset(_G,"CycleInit",function(player, selectedSkin)
	player.mo.color = selectedSkin.color[1]
end)

rawset(_G,"CycleThink",function(player, selectedSkin)
	player.mo.color = selectedSkin.color[((leveltime / tonumber(selectedSkin.cycle_speed)) % #selectedSkin.color)+1]
end)
--------------------------------------------------------------------------------
rawset(_G,"SpeedInit",function(player, selectedSkin)
	if type(selectedSkin.color[1]) == "table" then
		player.mo.color = selectedSkin.color[1][1]
	else
		player.mo.color = selectedSkin.color[1]
	end
end)

rawset(_G,"SpeedThink",function(player, selectedSkin)
	local speedtier = min(player.speed / 48550 / 15, 5) + 1
	if type(selectedSkin.color[speedtier]) == "table" then
		player.mo.color = selectedSkin.color[speedtier][((leveltime / tonumber(selectedSkin.cycle_speed)) % #selectedSkin.color[speedtier])+1]
	else
		player.mo.color = selectedSkin.color[speedtier]
	end
end)
--------------------------------------------------------------------------------
rawset(_G,"PositionInit",function(player, selectedSkin)
	if type(selectedSkin.color[player.kartstuff[k_position]]) == "table" then
		playermo.color = selectedSkin.color[p.kartstuff[k_position]][1]
	else
		player.mo.color = selectedSkin.color[player.kartstuff[k_position]]
	end
end)

rawset(_G,"PositionThink",function(player, selectedSkin)
	local curpos = player.kartstuff[k_position]
	if type(selectedSkin.color[curpos]) == "table"
		player.mo.color = selectedSkin.color[curpos][((leveltime / tonumber(selectedSkin.cycle_speed)) % #selectedSkin.color[curpos])+1]
	else
		player.mo.color = selectedSkin.color[player.kartstuff[k_position]]
	end
end)
--------------------------------------------------------------------------------
rawset(_G,"ItemInit",function(player, selectedSkin)
	player.mo.color = player.skincolor
	player.originalColor = player.skincolor
end)

rawset(_G,"ItemThink",function(player, selectedSkin)
	local curitem = player.kartstuff[k_itemtype]
	if curitem != 0
		if type(selectedSkin.color[curitem]) == "table" then
			player.mo.color = selectedSkin.color[curitem][((leveltime / tonumber(selectedSkin.cycle_speed)) % #selectedSkin.color[curitem])+1]
		else
			player.mo.color = selectedSkin.color[curitem]
		end
	else
		player.mo.color = player.originalColor
	end
end)